/* libing - collection of usefull objects
(c) 2002,2003
Stanislav Ievlev <inger@altlinux.org>
*/

#ifndef ING_COLOR_H
#define ING_COLOR_H

#include <unistd.h>
#include <term.h>

#include <string>
#include <map>

#include <libing/global.h>

//some definitions from ncurses header, move here to reduce deps on ncurses: use terminfo only
#define NCURSES_ATTR_SHIFT       8
#define NCURSES_BITS(mask,shift) ((mask) << ((shift) + NCURSES_ATTR_SHIFT))

#define A_NORMAL	0L
#define A_ATTRIBUTES	NCURSES_BITS(~(1UL - 1UL),0)
#define A_CHARTEXT	(NCURSES_BITS(1UL,0) - 1UL)
#define A_COLOR		NCURSES_BITS(((1UL) << 8) - 1UL,0)
#define A_STANDOUT	NCURSES_BITS(1UL,8)
#define A_UNDERLINE	NCURSES_BITS(1UL,9)
#define A_REVERSE	NCURSES_BITS(1UL,10)
#define A_BLINK		NCURSES_BITS(1UL,11)
#define A_DIM		NCURSES_BITS(1UL,12)
#define A_BOLD		NCURSES_BITS(1UL,13)
#define A_ALTCHARSET	NCURSES_BITS(1UL,14)
#define A_INVIS		NCURSES_BITS(1UL,15)
#define A_PROTECT	NCURSES_BITS(1UL,16)
#define A_HORIZONTAL	NCURSES_BITS(1UL,17)
#define A_LEFT		NCURSES_BITS(1UL,18)
#define A_LOW		NCURSES_BITS(1UL,19)
#define A_RIGHT		NCURSES_BITS(1UL,20)
#define A_TOP		NCURSES_BITS(1UL,21)
#define A_VERTICAL	NCURSES_BITS(1UL,22)

namespace Ing
{

//main class to make color output
class Color
{
	enum colors {BLACK, RED, GREEN, YELLOW, BLUE, MAGENTA, CYAN, WHITE};

	typedef unsigned long ncolor_t;
public:
	Color();
	~Color();
	void set_fg(const std::string& color_name, const std::string& outstream);
	void reset(const std::string& outstream);
#ifdef USE_COLOR_STREAMS
	std::ostream& set_fg(const std::string& color_name, std::ostream& out);
	void reset(std::ostream& out);
#endif
private:
	//sub-constructor
	void init_cache();

	//internal implementation
	ncolor_t split(const std::string& color_name);
	void set_attr(ncolor_t attr, int f(int));
	void reset(int f(int));
	bool real_stream(const std::string& outstream);
#ifdef USE_COLOR_STREAMS
	bool real_stream(int fd);
#endif

	//curses helpers
	static int stdout_putchar(int ch);
	static int stderr_putchar(int ch);
#ifdef USE_COLOR_STREAMS
	static int cout_putchar(int ch);
	static int cerr_putchar(int ch);
#endif
	static int toggled_colors(int c);

	//color semaphores
	bool	real_stdout;
	bool	real_stderr;
	bool	colorize;

	//color translation cache
	std::map<std::string, ncolor_t> cache;
};

#ifdef USE_COLOR_STREAMS
std::ostream& red(std::ostream& os);
std::ostream& green(std::ostream& os);
std::ostream& yellow(std::ostream& os);
std::ostream& blue(std::ostream& os);
std::ostream& standart(std::ostream& os);
#endif

std::ostream& empty(std::ostream& os);

};

#endif
