/* libing - collection of usefull objects
(c) 2002,2003
Stanislav Ievlev <inger@altlinux.org>
*/

#ifndef ING_ERR_H
#define ING_ERR_H

#include <cerrno>
#include <string>
#include <stdexcept>

namespace Ing
{

class glibc_error: public std::runtime_error
{
public:
	explicit
	glibc_error(const std::string& func, const std::string& name, const std::string& path = ""):
	std::runtime_error(func + " : " + name + " : " + path + " : " + strerror(errno)) {};
};


class internal_error: public std::runtime_error
{
public:
	explicit
	internal_error(const std::string& func, const std::string& descr):
	std::runtime_error(func + " : " + descr) {};
};

class empty_parameter: public internal_error
{
public:
	explicit
	empty_parameter(const std::string& func):
	internal_error(func, "empty parameter") {};
};


};

//usefull macros
#ifdef __GNUC__
 #define GLIBC_ERROR3(func,name,path) Ing::glibc_error(__PRETTY_FUNCTION__,name,path)
#else
 #define GLIBC_ERROR3(func,name,path) Ing::glibc_error(func,name,name,path)
#endif

#ifdef __GNUC__
 #define GLIBC_ERROR2(func,name) Ing::glibc_error(__PRETTY_FUNCTION__,name)
#else
 #define GLIBC_ERROR2(func,name) Ing::glibc_error(func,name,name)
#endif

#ifdef __GNUC__
 #define INTERNAL_ERROR(func,problem) Ing::internal_error(__PRETTY_FUNCTION__,problem)
#else
 #define INTERNAL_ERROR(func,problem) Ing::internal_error(func,problem)
#endif

#ifdef __GNUC__
 #define EMPTY_PARAMETER(func) Ing::empty_parameter(__PRETTY_FUNCTION__)
#else
 #define EMPTY_PARAMETER(func) Ing::empty_parameter(func)
#endif

#endif
